@@ -14,6 +14,7 @@
 # 2015-04-08: added setStrip(), restoreStrip(). J. Fox
 # 2015-07-07: fixed matchVarName() so that it handles periods in names properly. J. Fox
 # 2015-09-10: added a fix for class = 'array' in Analyze.model.  S. Weisberg
+# 2016-02-16: fix Analyze.model(), Fixup.model.matrix() to handle non-focal terms like polynomials correctly; clean up code. J. Fox
 
 has.intercept <- function(model, ...) any(names(coefficients(model))=="(Intercept)")
 
@@ -205,8 +206,7 @@ vcov.eff <- function(object, ...) object$vcov
 ### the following functions are for use by Effect() methods
 
 Analyze.model <- function(focal.predictors, mod, xlevels, default.levels=NULL, formula.rhs, 
-    partial.residuals=FALSE, quantiles, x.var=NULL, data=NULL){
-  #browser()
+    partial.residuals=FALSE, quantiles, x.var=NULL, data=NULL, typical=mean){
     if ((!is.null(mod$nan.action)) && class(mod$na.action) == "exclude") 
         class(mod$na.action) <- "omit"
     all.predictors <- all.vars(formula.rhs)
@@ -272,7 +272,7 @@ Analyze.model <- function(focal.predictors, mod, xlevels, default.levels=NULL, f
         x.var <- which(numeric.predictors)[1]
         x.var.name <- focal.predictors[x.var]
         if (is.null(mod$xlevels[[x.var.name]])){
-          x.var.levels <- x[[x.var]][["levels"]]
+#          x.var.levels <- x[[x.var]][["levels"]]
           x.var.range <- range(X[, focal.predictors[x.var]])
           x[[x.var]][["levels"]] <- seq(from=x.var.range[1], to=x.var.range[2], length=100)
         }
@@ -282,7 +282,7 @@ Analyze.model <- function(focal.predictors, mod, xlevels, default.levels=NULL, f
     for (name in excluded.predictors){
         levels <- mod$xlevels[[name]]
         fac <- !is.null(levels)
-        level <- if (fac) levels[1] else mean(X[, name])
+        level <- if (fac) levels[1] else typical(X[, name])
         if (fac) factor.levels[[name]] <- levels
         x.excluded[[name]] <- list(name=name, is.factor=fac,
             level=level)
@@ -398,10 +398,10 @@ Fixup.model.matrix <- function(mod, mod.matrix, mod.matrix.all, X.mod,
     factor.cols, cnames, focal.predictors, excluded.predictors, 
     typical, given.values,
     partial.residuals=FALSE, mod.matrix.all.rounded){
-    vars <- as.character(attr(terms(mod), "variables"))[-(1:2)]
+#    vars <- as.character(attr(terms(mod), "variables"))[-(1:2)]
     attr(mod.matrix, "assign") <- attr(mod.matrix.all, "assign")
     if (length(excluded.predictors) > 0){
-        sel <- apply(sapply(excluded.predictors, matchVarName, expressions=vars), 1, any)
+#        sel <- apply(sapply(excluded.predictors, matchVarName, expressions=vars), 1, any)
         strangers <- Strangers(mod, focal.predictors, excluded.predictors)
         stranger.cols <-  
             apply(outer(strangers, attr(mod.matrix,'assign'), '=='), 2, any)
@@ -410,7 +410,7 @@ Fixup.model.matrix <- function(mod, mod.matrix, mod.matrix.all, X.mod,
     if (has.intercept(mod)) stranger.cols[1] <- TRUE
     if (any(stranger.cols)) {
         facs <- factor.cols & stranger.cols
-        covs <- (!factor.cols) & stranger.cols
+#        covs <- (!factor.cols) & stranger.cols
         if (any(facs)){ 
             mod.matrix[,facs] <-  matrix(apply(as.matrix(X.mod[,facs]), 2, mean), 
                 nrow=nrow(mod.matrix), ncol=sum(facs), byrow=TRUE)
@@ -419,14 +419,14 @@ Fixup.model.matrix <- function(mod, mod.matrix, mod.matrix.all, X.mod,
                     matrix(apply(as.matrix(X.mod[,facs]), 2, mean), nrow=nrow(mod.matrix.all), ncol=sum(facs), byrow=TRUE)
             }
         }
-        if (any(covs)){ 
-            mod.matrix[,covs] <- matrix(apply(as.matrix(X.mod[,covs]), 2, typical), 
-                nrow=nrow(mod.matrix), ncol=sum(covs), byrow=TRUE)
-            if (partial.residuals) {
-                mod.matrix.all.rounded[,covs] <- mod.matrix.all[,covs] <- 
-                    matrix(apply(as.matrix(X.mod[,covs]), 2, typical), nrow=nrow(mod.matrix.all), ncol=sum(covs), byrow=TRUE)
-            }
-        }
+        # if (any(covs)){ 
+        #     mod.matrix[,covs] <- matrix(apply(as.matrix(X.mod[,covs]), 2, typical), 
+        #         nrow=nrow(mod.matrix), ncol=sum(covs), byrow=TRUE)
+        #     if (partial.residuals) {
+        #         mod.matrix.all.rounded[,covs] <- mod.matrix.all[,covs] <- 
+        #             matrix(apply(as.matrix(X.mod[,covs]), 2, typical), nrow=nrow(mod.matrix.all), ncol=sum(covs), byrow=TRUE)
+        #     }
+        # }
         if (!is.null(given.values)){
             stranger.names <- cnames[stranger.cols]
             given <- stranger.names %in% names(given.values)
